/*
 * Copyright (C) 2005-2006  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef FILETOOLS_H
# define FILETOOLS_H

# include <string>
# include <fstream>
# include "filetools_cfg.h"

class FileTools
{
public:
  
  /**
   * Creates a relative path
   */
  static std::string createRelPath(std::string refPath,
                                   std::string absPath);
  
  /** 
   * Creates an absolute path from rel and base.
   */
  static std::string createAbsPath(std::string base,
                                   std::string rel);
  
  /**
   * Does the file exist?
   */
  static bool fileExists(std::string path);
  
  /**
   * Öffnet eine Datei zum Schreiben. Wenn es die Datei vorher schon gab,
   * wird sie unter name_backup gespeichert. name_backup wird dabei 
   * überschrieben, wenn es schon vorhanden ist.
   */
  static void openAndCreateBackup(std::ofstream& out, std::string path);
  
  /**
   * Compares ref to result.
   * 
   * 1:  ok
   * -1: there's a file, but it is too old
   * -2: there isn't even a file...
   */
  static int isFileUpToDate(std::string ref, std::string result);
  
  /**
   * returns 0 if everything is fine
   */
  static int makeSureDirExists(std::string dirname);
  
  /**
   * Versucht $HOME/.appdirname (linux) oder
   * $USERPROFILE/appdirname zu öffnen oder anzulegen.
   * Wenn das klappt, wird dieses Verzeichnis zurückgegeben,
   * ansonsten das aktuelle Verzeichnis.
   * Das zurückgegebene Verzeichnis endet mit einem "/".
   */
  static std::string getHomeDir(std::string appdirname);
  
  /**
   * Delete file.
   * Returns nonzero on error.
   */
  static int rm(std::string filename);
  
  /**
   * Get size of file (in bytes). Returns -1 on error.
   */
  static int getSize(std::string filename);
  
  /**
   * Returns name and suffix of a file, strips leading directory information.
   */
  static std::string name(std::string pathAndName);
  
  /**
   * Returns path (with trailing '/', unless there is no path) of a file, 
   * strips name and suffix.
   */
  static std::string path(std::string pathAndName);
  
  /**
   * Removes everything after the first dot.
   */
  static std::string removeSuffix(std::string name);
  
  /**
   * Returns a string without 'difficult' characters.
   * Only numbers, a-z, underscore are allowed.
   * One may add characters which are allowed on any filesystem
   * without quoting.
   * '/' and '\' must not be included.    
   */
  static std::string cleanString(std::string name, std::string allowed = "");
  
  /**
   * Copies a file. 
   * dest can be a file or a directory (ends with '/').
   */
  static int copy(std::string dest, std::string src);
  
  /**
   * Results in a path with trailing '/', but no leading things.
   * It is used to concat paths.
   */
  static std::string stripAbsFromPath(std::string in);
  
  static bool isDir(std::string name);

  static int symlink(std::string dest, std::string src);
  
};


/**
 * A class to create a temporary text file and write text to it. 
 * It returns the filename to enable reading it.
 * This is far from perfect at least on one platform.
 * 
 * The main reason to write it was to achieve some portability and
 * a mostly stable interface.
 */
class TemporaryTextFile
{
public:

  static std::string GetGlobalTempDirPath();
  
  bool isOK() { return(fOK); };
  
  std::string getPathAndName() { return(FilenameAndPath); };
  
# if (TEMP_TEXTFILE_VARIANT == 0)
  int filedescr;
  
  /**
   * Uses mkstemp. "man mkstemp" advises to use "man 3 tmpfile". But how to 
   * get the filename in this case?
   */
  TemporaryTextFile(std::string name_template);
# endif
  
# if (TEMP_TEXTFILE_VARIANT == 1)
  std::ofstream outfile;
  
  /**
   * Poor man's method
   */
  TemporaryTextFile(std::string name_template);
# endif
  
# if (TEMP_TEXTFILE_VARIANT == 2)
  FILE*        tmpfiledescr;
  
  /**
   * using "man 3 tmpfile"
   */
  TemporaryTextFile();
# endif
  
  /**
   * Write some characters to the file
   */
  void write(std::string line);
  
  /**
   * Close the file, but don't delete it!
   */
  void close();
  
private:
  
  std::string FilenameAndPath;
  
  bool fOK;
};

#endif
