/*
 * Copyright (C) 2004, 2007  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "get.h"

#include <unistd.h>
#include <sys/wait.h>
#include <stdlib.h>
#include <iostream>
#include <string>
#include <dirent.h>     // fr die Verzeichnisoperationen
#include <sys/stat.h>   // Abfragen der Dateiattribute
#include <sys/types.h>

#include "tools.h"

/**
 * 
 */
SiVeCoGet::SiVeCoGet(SimpleXMLTransfer* ptr_conf,
                     std::string        workingDir,
                     eCmd               i_cmd)
{
  conf = ptr_conf;
  wd   = workingDir;  
  cmd  = i_cmd;
}

/**
 * Get version
 */
int SiVeCoGet::run(int         nNr,
                   std::string type, 
                   bool        fQuiet)
{
  std::string        infilename;
  SimpleXMLTransfer* links;
  
  if (nNr == -1)
    nNr = getCurrentVersionNumberInt(conf);
  
  // Check if repository exists
  rd = getConfDir(conf, "repository") + getVersionNumberString(nNr);
  if (existDir(rd) == false)
  {
    std::cout << "Error: repository " << rd << " does not exist.\n";
    return(-1);
  }
  infilename = rd + ".xml";
  if (existDat(infilename) == false)
  {
    std::cout << "Error: repository file " << infilename << " does not exist.\n";
    return(-1);
  }
  try
  {
    logfile = new SimpleXMLTransfer(infilename);
    // Die Version berprfen
    if (canHandleFileVersion(logfile->attributeAsInt("version")) == false)
    {
      std::cout << "Can't handle that version!\n";
      delete logfile;
      return(-1);      
    }
    // mich interessieren nur die Links:
    links = logfile->getChild("links");
  }
  catch (XMLException e)
  {
    std::cout << e.what() << "\n";
    return(-1);
  }
  
  if (cmd == CMD_ARCHIVE)
    wd += conf->attribute("name") + "-" + logfile->getChild("label")->getContentString() + "/";
  
  // Check if working directory exists
  if (existDir(wd) == true)
  {
    std::cout << "Error: directory " << wd << " exists.\n";
    std::cout << "       I won't write into an existing directory.\n";
    std::cout << "       Delete it if you are sure that you don't need\n";
    std::cout << "       its contents anymore or tell me to put things\n";
    std::cout << "       into another directory.\n";
    delete logfile;
    return(-1);
  }
  
  // Create working directory
  if (mkdir(wd.c_str(), S_IRWXU) != 0)
  {
    std::cout << "Error when creating directory: " << wd << "\n";
    delete logfile;
    return(-1);
  }  
  
  rd += "/";
  
  if (!fQuiet)
  {
    std::cout << "Getting\n";
    std::cout << "    " << rd << "\n";
    std::cout << "to\n";
    std::cout << "    " << wd << "\n";   
  }
  
  
  {
    {
      charsetconv = (iconv_t)-1;
      
      std::string charset_src_default = conf->getString("repo.charset.filenames", "");
      std::string charset_src         = logfile->getString("charset.filenames", charset_src_default);
      std::string charset_dst         = conf->getString("sys.charset.filenames", "");
      
      if (charset_src.length() != 0 && charset_dst.length() != 0 && charset_src.compare(charset_dst) != 0)
      {
        // Zeichenstze sind verschieden, es ist eine Wandlung ntig
        charsetconv = iconv_open(charset_src.c_str(), charset_dst.c_str());
        if (charsetconv == (iconv_t)-1)
        {
          std::cerr << "Error initializing characterset conversion: " << charset_src << " -> " << charset_dst << "\n";
          return(-1);
        }
      }
    }
    
    int retcode = get("", links);
    
    if (charsetconv != (iconv_t)-1)
      iconv_close(charsetconv);
    
    if (retcode)
    {
      delete logfile;
      return(-1);
    }
  }
  
  switch (cmd)
  {
    case CMD_GET:
      if (type.length() == 0)
      {
        // Notieren, welche Version abgerufen wurde
        if (setDerivedFrom(conf, wd, nNr))
          return(-1);
        
        // afterGetCmd mit der Nummer der gerade abgerufenen Version
        if (conf->attribute("afterGetCmd").length())
          executeCmd(wd + conf->attribute("afterGetCmd"), itoStr(nNr, ' ', 0));
        
        // Eine leere Changelog-Datei erstellen
        if (conf->attribute("changelog").length())
          if (createEmptyChangeLog(wd + conf->attribute("changelog")))
            return(-1);
      }
      break;
      
    case CMD_ARCHIVE:
      // archiveCmd mit der Nummer der gerade abgerufenen Version
      if (conf->attribute("archiveCmd").length())
        executeCmd(wd + conf->attribute("archiveCmd"), itoStr(nNr, ' ', 0));
      break;
      
    default:
      break;
  }
  
  delete logfile;
  
  return(0);
}


int SiVeCoGet::get(std::string         dir,
                   SimpleXMLTransfer*  links)
{
  DIR*            Verz;                 // der VerzeichnisStrom
  dirent*         VerzEintr;            // hier werden die Daten aus obigem gegeben
  struct stat     Attribs;              // ein Puffer fr die Dateiattribute
  std::string     currentDir  = rd;
  
  currentDir.append(dir);
  
  
  Verz = opendir(currentDir.c_str());
  if (Verz != 0)
  {    
    // Rekursive Schleife ber alle Dinge, die ich im repository finde:
    while ((VerzEintr = readdir(Verz)) != NULL)
    {
      std::string currentFile = currentDir;
      currentFile += VerzEintr->d_name;
      lstat(currentFile.c_str(), &Attribs);
      
      std::string some_name = VerzEintr->d_name;
      if (charsetconv != (iconv_t)-1)
      {
        char*  inbuf        = VerzEintr->d_name;
        size_t inbytesleft  = strlen(VerzEintr->d_name);
        size_t outbytesleft = 4*inbytesleft+1;
        char*  outbuf       = (char*)malloc(outbytesleft);
        char*  wrptr        = outbuf;
        
        if (outbuf == 0)
        {
          std::cout << "Error on charset conversion\n";
          return(-1);
        }
        
        iconv(charsetconv, 
              0, 0,
              &outbuf, &outbytesleft);
        iconv(charsetconv, 
              &inbuf, &inbytesleft,
              &wrptr, &outbytesleft);
        *wrptr = '\0';
        some_name = outbuf;
        free(outbuf);
        
        if (inbytesleft != 0)
        {
          std::cout << "Error on charset conversion: " << VerzEintr->d_name << "\n";
          return(-1);
        }
      }
      
      // der relative Pfad dieses Elements
      std::string relName = dir + VerzEintr->d_name;
      // Unter diesem Pfad muss die Datei gesichert werden:
      std::string wfile = wd + relName;
      
      if (S_ISLNK(Attribs.st_mode) != 0)
      {
        // Ein Link. Gibt es dieses in links?
        SimpleXMLTransfer* tmp    = (SimpleXMLTransfer*)0;
        int                maxnum = links->getChildCount();
        int                cnt    = 0;
        for (cnt=0; cnt<maxnum; cnt++)
        {
          tmp = links->getChildAt(cnt);
          if (tmp->getName().compare("link") == 0 &&
              tmp->attribute("name").compare(some_name) == 0)
            cnt = maxnum+1;
        }
        
        if (cnt != maxnum+2)
        {
          // nicht vorhanden, also dem Link folgen und kopieren
          std::string command;
          
          command = "cp -p -L \"" + currentFile + "\" \"" + wfile + "\"";
          system(command.c_str());                    
        }
        else if (tmp->attributeAsInt("abs") == 0)
        {
          // Relatives Link. Also den absoluten Pfad aus dem Repository in
          // einen relativen umwandeln oder relativ lassen.
          std::string target = getLinkTarget(currentFile);
          
          if (target.length() == 0)
          {
            std::cout << "Error\n";
            return(-1);
          }
          
          if (target[0] == '/')
          {
            if (symlink(getRelPath(wd+dir, target).c_str(), wfile.c_str()) != 0)
            {
              std::cout << "Error when creating symlink\n";
              return(-1);
            }        
          }
          else
          {
            if (symlink(target.c_str(), wfile.c_str()) != 0)
            {
              std::cout << "Error when creating symlink\n";
              return(-1);
            }        
          }
        }
        else
        {
          // Absolutes Link. Weil es auch absolut im Repository gespeichert ist, 
          // bedeutet das wenig Arbeit.
          std::string command;
          
          command = "cp -p -d \"" + currentFile + "\" \"" + wfile + "\"";
          system(command.c_str());                              
        }
        
      }
      else
      {
        if (S_ISDIR(Attribs.st_mode) != 0)
        {
          if (strcmp("..", VerzEintr->d_name) != 0 && strcmp(".", VerzEintr->d_name) != 0)
          {
            // Normales Verzeichnis. Einfach erzeugen und darin weitermachen.
            if (mkdir(wfile.c_str(), S_IRWXU) != 0)
            {
              std::cout << "Error when creating directory\n";
              return(-1);
            }
            
            // auch in den notierten Links abtauchen
            SimpleXMLTransfer* tmp    = (SimpleXMLTransfer*)0;
            int                maxnum = links->getChildCount();
            int                cnt    = 0;
            for (cnt=0; cnt<maxnum; cnt++)
            {
              tmp = links->getChildAt(cnt);
              
              if (tmp->getName().compare("dir") == 0 &&
                  tmp->attribute("name").compare(some_name) == 0)
                cnt = maxnum+2;
            }
            
            if (cnt < maxnum+2)
            {
              std::cout << "Error: unable to find " << relName << " in logfile!\n";
              std::cout << "*" << some_name << "*\n";
              links->print();
              return(-1);
            }
            
            if (get(relName+"/", tmp) != 0)
            {
              std::cout << "Error\n";
              return(-1);
            }
          }
        }
        else
        {
          // Normale Datei. Kopieren.
          std::string command;
          
          command = "cp -p \"" + currentFile + "\" \"" + wfile + "\"";
          system(command.c_str());                    
        }          
      }
    }
    closedir(Verz);
    return(0);
  }
  else
    return(-1);  
}
