/*
 * Copyright (C) 2004, 2007  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "SimpleXMLTransfer.h"
#include "create.h"
#include "get.h"
#include "remove.h"
#include "move.h"
#include "info.h"
#include <iostream>
#include <stdlib.h>
#include <string>

#include "version.h"
#include "types.h"
#include "tools.h"

/**
 * Print usage
 */
void printUsage()
{
  std::cout << "SiVeCo -- Simple Version Control              Version " << strProjectVersion << "\n\n";
  
  std::cout << "Here's just a short reference. See the documentation for details.\n\n";
  std::cout << "     siveco command configfile [options]\n\n";
  
  std::cout << "Create new version:\n";
  std::cout << "     siveco c configfile [-n number] [-d directory] [-l label]\n";
  std::cout << "Get a version:\n";
  std::cout << "     siveco g configfile [-q] [-n number] [-d directory] [-t archive]\n";
  std::cout << "Compare working directory with saved version (show diff):\n";
  std::cout << "     siveco d configfile [-n number] [-d directory] [-t type]\n";
  std::cout << "  type: list (default), exec, bexec.\n";
  std::cout << "Remove versions from the repository:\n";
  std::cout << "     siveco r configfile -n number\n";
  std::cout << "     siveco r configfile -n numberStart-numberEnd\n";
  std::cout << "Get Information:\n";
  std::cout << "     siveco i configfile [-n number] [-t type]\n";
  std::cout << "  type: history (default), historyrev,\n";
  std::cout << "        historyhtm, historyhtmrev,\n";
  std::cout << "        historyhtmNH, historyhtmNHrev,\n";
  std::cout << "        label, nameLabel, changes.\n";
  std::cout << "Create archive:\n";
  std::cout << "     siveco a configfile [-n number]\n";
  std::cout << "Create script to convert(move) into another VCS\n";
  std::cout << "     siveco m configfile\n";
}

/**
 * Simple main function
 */
int main(int argc, const char* argv[])
{
  eCmd               cmd = CMD_CREATE;
  SimpleXMLTransfer* conf;
  bool               fArgsOK = true;
  std::string        conffilepath = "";
  std::string        workingdir   = "";
  std::string        type         = "";
  std::string        label        = "";
  int                nNr          = -1;
  int                nNrEnd       = -1;
  int                retCode      = 0;
  bool               fQuiet       = false;
  
  
  // --- Argumente parsen -------------------------------------------
  if (argc < 3          ||    // at least two arguments
      strlen(argv[1]) != 1)   // length of first argument
    fArgsOK = false;
  else
  {
    int cnt=3;
    
    switch (argv[1][0])
    {
      case 'c':
        cmd = CMD_CREATE;
        conffilepath = argv[2];
        break;
      case 'm':
        cmd = CMD_MOVE;
        conffilepath = argv[2];
        break;
      case 'g':
        cmd = CMD_GET;
        conffilepath = argv[2];
        break;
      case 'a':
        cmd = CMD_ARCHIVE;
        conffilepath = argv[2];
        break;
      case 'd':
        cmd = CMD_DIFF;
        conffilepath = argv[2];
        break;
      case 'r':
        cmd = CMD_REMOVE;
        conffilepath = argv[2];
        break;
      case 'i':
        cmd = CMD_INFO;
        conffilepath = argv[2];
        type = "history";
        break;
      default:
        fArgsOK = false;
        break;
    }
    
    while (cnt < argc && fArgsOK)
    {
      if (strlen(argv[cnt]) == 2 && argv[cnt][0] == '-')
      {
        switch (argv[cnt][1])
        {
          case 'n': // Number
            {
              std::string tmp = argv[cnt+1];
              
              if (tmp.find('-') != std::string::npos && tmp.find('-') != 0)
              {
                nNr    = atoi(tmp.substr(0, tmp.find('-')).c_str());
                nNrEnd = atoi(tmp.substr(tmp.find('-')+1).c_str());
                
                if (nNrEnd < nNr)
                {
                  int tmp;
                  
                  tmp    = nNrEnd;
                  nNrEnd = nNr;
                  nNr    = tmp;
                }
              }
              else
              {
                nNr    = atoi(argv[cnt+1]);
                nNrEnd = nNr;
              }
              
              cnt+= 2;
            }
            break;
          case 'd': // Working directory
            workingdir = argv[cnt+1];
            cnt+= 2;
            break;
          case 't': // Type
            type = argv[cnt+1];
            cnt+= 2;
            break;
          case 'q': // quiet
            fQuiet = true;
            cnt+= 1;
            break;
          case 'l': // Label
            label = argv[cnt+1];
            cnt+= 2;
            break;
          default:  // unknown argument
            std::cerr << "Unknown argument: " << argv[cnt] << "\n";
            fArgsOK = false;
            break;
        }
      }
      else
      {
        std::cerr << "Unexpected argument: " << argv[cnt] << "\n";
        fArgsOK = false;
      }
    }    
  }
  
  if (fArgsOK == false)
  {
    printUsage();
    return(-1);
  }
  
  // --- Argumente prfen -------------------------------------------
  try
  {
    conf = new SimpleXMLTransfer(conffilepath.c_str());
  }
  catch (XMLException e)
  {
    std::cout << e.what() << "\n";
    return(-1);
  }
  
  if (workingdir.length() == 0)
  {
    switch (cmd)
    {
      case CMD_ARCHIVE:
        workingdir = getConfDir(conf, "archiveDirectory");
        break;
      default:
        workingdir = getConfDir(conf, "workingDirectory");
        break;
    }
  }
  
  if (workingdir.length() == 0)
  {
    std::cout << "Error: No workingDirectory/archiveDirectory given\n";
    return(-1);
  }
  
  if (workingdir[0] != '/')
  {
    std::cout << "Error: workingDirectory/archiveDirectory not absolute\n";
    return(-1);
  }
  
  if (conf->attribute("repository").length() == 0)
  {
    std::cout << "Error: No repository given\n";
    return(-1);
  }
  
  if (conf->attribute("repository")[0] != '/')
  {
    std::cout << "Error: repository not absolute\n";
    return(-1);
  }
  
  // --- Aktion ausfhren -------------------------------------------
  switch (cmd)
  {
    case CMD_CREATE:
    case CMD_DIFF:    
      {
        SiVeCoCreate* cr = new SiVeCoCreate(conf, workingdir, cmd);
        if (cr->run(nNr, type, label))
          retCode = -1;
        delete cr;
      }
      break;
      
    case CMD_ARCHIVE:
    case CMD_GET:
      {
        SiVeCoGet* get = new SiVeCoGet(conf, workingdir, cmd);
        if (get->run(nNr, type, fQuiet))
          retCode = -1;
        delete get;
      }
      break;
      
    case CMD_MOVE:
      {
        SiVeCoMove* move = new SiVeCoMove(conffilepath, conf);
        if (move->run())
          retCode = -1;
        delete move;
      }
      break;
      
    case CMD_REMOVE:
      {
        if (nNr == -1)
        {
          std::cout << "Error: No version-nr given\n";
          return(-1);
        }
        SiVeCoRemove* rm = new SiVeCoRemove(conf);
        if (rm->run(nNr, nNrEnd))
          retCode = -1;
        delete rm;
      }
      break;
      
    case CMD_INFO:
      if (type.find("history") != std::string::npos)
      {
        if (printHistory(conf, type))
          retCode = -1;
      }
      else
      {
        if (printInfo(conf, nNr, type))
          retCode = -1;
      }
      break;
      
    default:
      break;
  }
  
  delete conf;
  
  if (retCode)
    std::cout << "An error occured.\n";
  
  return(retCode);
}
